@@ -1,7 +1,7 @@
 /*
  * fsarchiver: Filesystem Archiver
  *
- * Copyright (C) 2008-2009 Francois Dupoux.  All rights reserved.
+ * Copyright (C) 2008-2010 Francois Dupoux.  All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -15,7 +15,9 @@
  * Homepage: http://www.fsarchiver.org
  */
 
-#include "fsarchiver.h"
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <string.h>
 #include <stdio.h>
@@ -23,13 +25,15 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <uuid.h>
 
+#include "fsarchiver.h"
 #include "dico.h"
 #include "common.h"
 #include "fs_btrfs.h"
-#include "uuid.h"
 #include "filesys.h"
 #include "strlist.h"
+#include "error.h"
 
 int btrfs_check_compatibility(u64 compat, u64 incompat, u64 ro_compat)
 {
@@ -54,6 +58,7 @@ int btrfs_mkfs(cdico *d, char *partition)
     u64 compat_flags;
     u64 incompat_flags;
     u64 compat_ro_flags;
+    int exitst;
     u64 temp64;
     
     // ---- get original filesystem features (if the original filesystem was a btrfs)
@@ -73,7 +78,7 @@ int btrfs_mkfs(cdico *d, char *partition)
     }
     
     // ---- there is no option that just displays the version and return 0 in mkfs.btrfs-0.16
-    if (exec_command(command, sizeof(command), NULL, 0, NULL, 0, "which mkfs.btrfs")!=0)
+    if (exec_command(command, sizeof(command), NULL, NULL, 0, NULL, 0, "mkfs.btrfs")!=0)
     {   errprintf("mkfs.btrfs not found. please install btrfs-progs on your system or check the PATH.\n");
         return -1;
     }
@@ -86,7 +91,7 @@ int btrfs_mkfs(cdico *d, char *partition)
     if (dico_get_u64(d, 0, FSYSHEADKEY_FSBTRFSSECTORSIZE, &temp64)==0)
         strlcatf(options, sizeof(options), " -s %ld ", (long)temp64);
     
-    if (exec_command(command, sizeof(command), NULL, 0, NULL, 0, "mkfs.btrfs %s %s", partition, options)!=0)
+    if (exec_command(command, sizeof(command), &exitst, NULL, 0, NULL, 0, "mkfs.btrfs %s %s", partition, options)!=0 || exitst!=0)
     {   errprintf("command [%s] failed\n", command);
         return -1;
     }
@@ -97,7 +102,7 @@ int btrfs_mkfs(cdico *d, char *partition)
 int btrfs_getinfo(cdico *d, char *devname)
 {
     struct btrfs_super_block sb;
-    const char *str;
+    char uuid[512];
     u16 temp32;
     int ret=0;
     int fd;
@@ -131,9 +136,12 @@ int btrfs_getinfo(cdico *d, char *devname)
     dico_add_string(d, 0, FSYSHEADKEY_FSLABEL, (char*)sb.label);
     
     // ---- uuid
-    if ((str=e2p_uuid2str(sb.dev_item.fsid))!=NULL)
-        dico_add_string(d, 0, FSYSHEADKEY_FSUUID, str);
-    msgprintf(MSG_DEBUG1, "btrfs_uuid=[%s]\n", str);
+    /*if ((str=e2p_uuid2str(sb.dev_item.fsid))!=NULL)
+        dico_add_string(d, 0, FSYSHEADKEY_FSUUID, str);*/
+    memset(uuid, 0, sizeof(uuid));
+    uuid_unparse_lower((u8*)sb.dev_item.fsid, uuid);
+    dico_add_string(d, 0, FSYSHEADKEY_FSUUID, uuid);
+    msgprintf(MSG_DEBUG1, "btrfs_uuid=[%s]\n", uuid);
     
     // ---- sector size
     temp32=le32_to_cpu(sb.sectorsize);
@@ -150,7 +158,7 @@ int btrfs_getinfo(cdico *d, char *devname)
     }
     
     // ---- minimum fsarchiver version required to restore
-    dico_add_u64(d, 0, FSYSHEADKEY_MINFSAVERSION, FSA_VERSION_BUILD(0, 5, 9, 0));
+    dico_add_u64(d, 0, FSYSHEADKEY_MINFSAVERSION, FSA_VERSION_BUILD(0, 6, 4, 0));
     
 btrfs_read_sb_close:
     close(fd);
