@@ -90,7 +90,11 @@ enum {FSAERR_SUCCESS=0,           // success
       FSAERR_ENOENT=-4,           // entry not found
       FSAERR_ENDOFFILE=-5,        // end of file/queue
       FSAERR_WRONGTYPE=-6,        // wrong type of data
-      FSAERR_NOTOPEN=-7           // ressource has been closed
+      FSAERR_NOTOPEN=-7,          // resource has been closed
+      FSAERR_ENOSPC=-8,           // no space left on device
+      FSAERR_SEEK=-9,             // lseek64 error
+      FSAERR_READ=-10,            // read error
+      FSAERR_WRITE=-11            // write error
 };
 
 // -------------------------------- old errors codes ---------------------------------------------
